# -*- coding: utf-8 -*-

import sys

if sys.version_info[:2] >= (3, 10):
    from cryptography import x509
    from cryptography.x509.oid import NameOID
    from cryptography.hazmat.primitives import hashes, serialization
    from cryptography.hazmat.primitives.asymmetric import rsa
    from cryptography.hazmat.backends import default_backend
else:
    from OpenSSL import crypto


class WsaaCertificate(object):

    def __init__(self, key):
        self.key = key
        self.country_code = None
        self.state_name = None
        self.company_name = None
        self.company_vat = None

    def validate_values(self):
        """ Validamos que esten todos los campos necesarios seteados """
        values = vars(self)
        for value in values:
            if not values.get(value):
                raise AttributeError('Falta configurar alguno de los siguientes campos:\n'
                                     'Codigo de pais, Provincia, Nombre de la empresa o CUIT')

    def generate_certificate_request(self, hash_sign='sha256', ou='odoo'):

        self.validate_values()

        if sys.version_info[:2] >= (3, 10):
            # Cargamos la clave privada desde el PEM
            try:
                private_key = serialization.load_pem_private_key(
                    self.key.encode('utf-8'),
                    password=None,
                    backend=default_backend()
                )
            except Exception as e:
                raise ValueError("Formato de clave privada inválido") from e

            # Cargamos los datos del certificado
            subject = x509.Name([
                x509.NameAttribute(NameOID.COUNTRY_NAME, self.country_code),
                x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, self.state_name),
                x509.NameAttribute(NameOID.ORGANIZATION_NAME, self.company_name),
                x509.NameAttribute(NameOID.ORGANIZATIONAL_UNIT_NAME, ou),
                x509.NameAttribute(NameOID.COMMON_NAME, self.company_name),
                x509.NameAttribute(NameOID.SERIAL_NUMBER, 'CUIT {}'.format(self.company_vat)),
            ])

            # Validamos que el algoritmo de hash sea válido
            try:
                hash_algo = getattr(hashes, hash_sign.upper())()
            except AttributeError:
                raise ValueError("Algoritmo de hash inválido: " + hash_sign)

            # Creamos y firmamos el certificado
            csr = (
                x509.CertificateSigningRequestBuilder()
                .subject_name(subject)
                .sign(private_key, hash_algo, default_backend())
            )

            return csr.public_bytes(serialization.Encoding.PEM)

        else:
            # Utilizamos la libreria de crypto para generar el pedido de certificado
            req = crypto.X509Req()
            req.get_subject().C = self.country_code
            req.get_subject().ST = self.state_name
            req.get_subject().O = self.company_name
            req.get_subject().OU = ou
            req.get_subject().CN = self.company_name
            req.get_subject().serialNumber = 'CUIT {}'.format(self.company_vat)

            # Validamos el formato de la key
            key = crypto.load_privatekey(crypto.FILETYPE_PEM, self.key)
            self.key = crypto.dump_privatekey(crypto.FILETYPE_PEM, key)

            # Firmamos con la key y el hash el certificado
            req.set_pubkey(key)
            req.sign(key, hash_sign)

            return crypto.dump_certificate_request(crypto.FILETYPE_PEM, req)


class WsaaPrivateKey(object):

    def __init__(self, length=2048):
        self.length = length
        self.key = None

    def generate_rsa_key(self):
        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=self.length,
            backend=default_backend()
        )
        self.key = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()
        )



# -*- coding: utf-8 -*-
# Segun RG 2485 – Proyecto FE v2.8 - 12/09/2016

from zeep import Client, helpers

from .error import AfipError
from l10n_ar_api.afip_webservices import config
from .invoice import ElectronicInvoiceValidator


class WsctInvoiceDetails(object):
    """
    Se encarga de asignar los detalles de una invoice en
    los factories de WSFE.

    :param client: Cliente / Webservice.
    :param invoice: Objeto ElectronicInvoice para tomar los detalles.
    :param last_invoice_number: Ultimo numero de comprobante.
    """

    def __init__(self, client, invoice, last_invoice_number):
        self.client = client
        self.invoice = invoice
        self.last_invoice_number = last_invoice_number
        self.detail = None

    def get_details(self):
        """ Devuelve los detalles completos para esa factura """

        self._set_details()

        return self.detail

    def _set_details(self):
        """ Completa los detalles a enviar segun los datos del documento """

        self.detail = self._get_detail()
        self.detail.Concepto = self.invoice.concept
        self.detail.DocTipo = self.invoice.customer_document_type
        self.detail.DocNro = self.invoice.customer_document_number
        self.detail.CbteDesde = self.last_invoice_number+1
        self.detail.CbteHasta = self.last_invoice_number+1
        self.detail.CbteFch = self.invoice.document_date
        self.detail.ImpTotal = round(self.invoice.get_total_amount(), 2)
        self.detail.ImpTotConc = round(self.invoice.untaxed_amount, 2)
        self.detail.ImpNeto = round(self.invoice.taxed_amount, 2)
        self.detail.ImpOpEx = round(self.invoice.exempt_amount, 2)
        self.detail.ImpIVA = round(self.invoice.get_total_iva(), 2)
        self.detail.ImpTrib = round(self.invoice.get_total_tributes(), 2)
        if self.invoice.concept not in [2, 3]:
            self.detail.FchServDesde = ''
            self.detail.FchServHasta = ''
            # Si es factura de crédito hay que informar siempre fecha de vencimiento
            self.detail.FchVtoPago = '' if self.invoice.document_code not in [201, 206, 211] else self.invoice.payment_due_date
        else:
            self.detail.FchServDesde = self.invoice.service_from
            self.detail.FchServHasta = self.invoice.service_to
            # Para notas de crédito / débito FCE no hay que informar fecha de vencimiento de pago.
            self.detail.FchVtoPago = self.invoice.payment_due_date if self.invoice.document_code not in [202, 203, 207, 208, 212, 213] \
                else None
        self.detail.MonId = self.invoice.mon_id
        self.detail.MonCotiz = round(self.invoice.mon_cotiz, 6)
        self._set_iva()
        self._set_tributes()
        self._set_optionals()
        self._set_associated_documents()
        self._set_period()

    def _serialize_period(self, period_from, period_to):
        return self._get_period()(
            FchDesde=period_from,
            FchHasta=period_to
        )

    def _set_period(self):
        """ Agrega al detalle el periodo asociado """
        if self.invoice.period_from and self.invoice.period_to:
            self.detail.PeriodoAsoc = self._serialize_period(self.invoice.period_from, self.invoice.period_to)

    def _serialize_tribute(self, tribute):
        return self._get_tribute()(
            Id=tribute.document_code,
            BaseImp=round(tribute.taxable_base, 2),
            Alic=tribute.aliquot,
            Importe=round(tribute.amount, 2),
            Desc='Impuesto codigo {}'.format(tribute.document_code),
        )

    def _set_tributes(self):
        """ Agrega al detalle el array de tributos del documento """

        if self.invoice.array_tributes:
            self.detail.Tributos = self._get_tribute_array()([
                self._serialize_tribute(tribute) for tribute in self.invoice.array_tributes
            ])

    def _serialize_iva(self, iva):
        return self._get_iva()(
            Id=iva.document_code,
            BaseImp=round(iva.taxable_base, 2),
            Importe=round(iva.amount, 2)
        )

    def _serialize_optional(self, optional):
        return self._get_optional()(
            Id=optional.optional_id,
            Valor=optional.value,
        )

    def _serialize_associated_document(self, associated_document):
        return self._get_associated_document()(
            Tipo=associated_document.document_type,
            PtoVta=associated_document.point_of_sale,
            Nro=associated_document.number,
            Cuit=associated_document.cuit,
            CbteFch=associated_document.document_date
        )

    def _set_iva(self):
        """ Agrega al detalle el array de iva del documento """

        if self.invoice.array_iva:
            self.detail.Iva = self._get_iva_array()([
                self._serialize_iva(iva) for iva in self.invoice.array_iva
            ])

    def _set_associated_documents(self):
        if self.invoice.associated_documents:
            self.detail.CbtesAsoc = self._get_associated_document_array()([
                self._serialize_associated_document(associated) for associated in self.invoice.associated_documents
            ])

    def _set_optionals(self):
        if self.invoice.array_optionals:
            self.detail.Opcionales = self._get_optional_array()([
                self._serialize_optional(optional) for optional in self.invoice.array_optionals
            ])

    def _get_detail(self):
        return self.client.type_factory('ns0').FECAEDetRequest()

    def _get_associated_document_array(self):
        return self.client.get_type('ns0:ArrayOfCbteAsoc')

    def _get_optional_array(self):
        return self.client.get_type('ns0:ArrayOfOpcional')

    def _get_associated_document(self):
        return self.client.get_type('ns0:CbteAsoc')

    def _get_optional(self):
        return self.client.get_type('ns0:Opcional')

    def _get_iva(self):
        return self.client.get_type('ns0:AlicIva')

    def _get_iva_array(self):
        return self.client.get_type('ns0:ArrayOfAlicIva')

    def _get_tribute(self):
        return self.client.get_type('ns0:Tributo')

    def _get_tribute_array(self):
        return self.client.get_type('ns0:ArrayOfTributo')

    def _get_period(self):
        return self.client.get_type('ns0:Periodo')


class WsctOptional(object):

    def __init__(self, optional_id, value):
        self.optional_id = optional_id
        self.value = value


class WsctAssociatedDocument(object):

    def __init__(self, document_type, point_of_sale, number, cuit, document_date):
        self.document_type = document_type
        self.point_of_sale = point_of_sale
        self.number = number
        self.cuit = cuit
        self._document_date = None
        self.document_date = document_date

    def _parse_date(self, value):
        return value.strftime('%Y%m%d')

    @property
    def document_date(self):
        return self._document_date

    @document_date.setter
    def document_date(self, value):
        self._document_date = self._parse_date(value)


class Wsct(object):
    """
    Factura electronica.

    :param access_token: AccessToken - Token de acceso
    :param cuit: Cuit de la empresa
    :param homologation: Homologacion si es True
    :param url: Url de servicios para Wsfe
    """

    def __init__(self, access_token, cuit, homologation=True, url=None):
        if not url:
            self.url = config.service_urls.get('wsct_homologation') if homologation\
                else config.service_urls.get('wsct_production')
        else:
            self.url = url

        self.accessToken = access_token
        self.cuit = cuit
        self.auth_request = self._create_auth_request()

    def check_webservice_status(self):
        """ Consulta el estado de los webservices de AFIP."""

        res = Client(self.url).service.dummy()

        if hasattr(res, 'Errors'):
            raise AfipError.parse_error(res)
        if res.appserver != 'OK':
            raise Exception('El servidor de aplicaciones no se encuentra disponible. Intente mas tarde.')
        if res.dbserver != 'OK':
            raise Exception('El servidor de base de datos no se encuentra disponible. Intente mas tarde.')
        if res.authserver != 'OK':
            raise Exception('El servidor de auntenticacion no se encuentra disponible. Intente mas tarde.')

    def get_cotization(self, currency):
        """
        :param currency: Id de la moneda consultar
        :return: Cotización de la moneda para el día de hoy.
        """
        cotiz_response = Client(self.url).service.FEParamGetCotizacion(
            Auth=self.auth_request,
            MonId=currency,
        )
        if cotiz_response.Errors:
            raise AfipError().parse_error(cotiz_response)

        return cotiz_response.ResultGet.MonCotiz

    def show_error(self, response):
        if response.arrayErrores:
            raise AfipError.parse_error(response)

    def get_last_number(self, pos_number, document_type_number):
        """
        :param pos_number: Numero de punto de venta
        :param document_type_number: Numero del tipo de documento segun AFIP a consultar
        :return: Numero de ultimo comprobante autorizado para ese tipo
        """

        # FECompUltimoAutorizado(Auth: ns0:FEAuthRequest, PtoVta: xsd:int, CbteTipo: xsd:int) ->
        # FECompUltimoAutorizadoResult: ns0:FERecuperaLastCbteResponse

        # params = {
        #     'authRequest': self.auth_request,
        #     'numeroPuntoVenta': pos_number,
        #     'codigoTipoComprobante': document_type_number,
        # }
        last_number_response = Client(self.url).service.consultarUltimoComprobanteAutorizado(
            authRequest=self.auth_request,
            numeroPuntoVenta=pos_number,
            codigoTipoComprobante=document_type_number,
        )

        if last_number_response.arrayErrores:
            if last_number_response.arrayErrores.codigoDescripcion[0].codigo != 1002:
                raise AfipError().parse_error(last_number_response)
        if last_number_response.numeroComprobante == None:
            return 0
        return last_number_response.numeroComprobante

    def _validate_invoices(self, invoices):
        """
        Valida que los campos de la factura electronica sean validos

        :param invoices: Lista de Objetos Invoice, documentos a validar.
        """

        invoiceValidator = ElectronicInvoiceValidator()
        for invoice in invoices:
            invoiceValidator.validate_invoice(invoice)

    def _get_header(self):
        return Client(self.url).get_type('ns0:autorizarComprobanteRequest')

    def _get_cae_request(self):
        return Client(self.url).get_type('ns0:FECAERequest')

    def _get_array_cae_request(self):
        return Client(self.url).get_type('ns0:ArrayOfFECAEDetRequest')

    def _get_document_type(self, invoice):
        return invoice.document_code

    def get_cae(self, invoices, pos):
        """
        :param invoices: Conjunto de Objetos ElectronicInvoice, documentos a enviar a AFIP.
        :param pos: Numero de punto de venta.
        :returns str: Respuesta de AFIP sobre la validacion del documento.
        """

        self._validate_invoices(invoices)
        data = self._set_wsct(invoices[0], pos)
        response = Client(self.url).service.autorizarComprobante(
            authRequest=data['authRequest'],
            comprobanteRequest=data['comprobanteRequest']
        )
        return response, data

    def _set_wsct(self, invoice, pos):
        """
        :param invoices: Conjunto de objetos ElectronicInvoice.
        :param pos: Numero de punto de venta.
        :returns: FECAERequest / Envio de documentos para recibir el CAE.
        """
        item = list()
        item_ = Client(self.url).get_type('ns0:ItemType')
        for itemdict in invoice.Items:
            item.append(item_(
                tipo=itemdict['tipo'],
                codigoTurismo=itemdict['codigoTurismo'],
                codigo=itemdict['codigo'],
                descripcion=itemdict['descripcion'],
                codigoAlicuotaIVA=itemdict['codigoAlicuotaIVA'],
                importeIVA=itemdict['importeIVA'],
                importeItem=itemdict['importeItem']
            ))

        ArrayItems_ = Client(self.url).get_type('ns0:ArrayItemsType')
        ArrayItems = ArrayItems_(
            item=item
        )

        Arrayasoc = None
        if invoice.arrayComprobantesAsociados:
            asoc = list()
            asoc_ = Client(self.url).get_type('ns0:ComprobanteAsociadoType')
            for itemdict in invoice.arrayComprobantesAsociados:
                asoc.append(asoc_(
                    codigoTipoComprobante=itemdict['codigoTipoComprobante'],
                    numeroPuntoVenta=itemdict['numeroPuntoVenta'],
                    numeroComprobante=itemdict['numeroComprobante'],
                ))

            Arrayasoc_ = Client(self.url).get_type('ns0:ArrayComprobantesAsociadosType')
            Arrayasoc = Arrayasoc_(
                comprobanteAsociado=asoc
            )


        Arrayothes = None
        if invoice.arrayOtrosTributos:
            others = list()
            others_ = Client(self.url).get_type('ns0:OtroTributoType')
            for itemdict in invoice.arrayOtrosTributos:
                others.append(others_(
                    codigo=itemdict['codigo'],
                    descripcion=itemdict['descripcion'],
                    baseImponible=itemdict['baseImponible'],
                    importe=itemdict['importe'],
                ))

            Arrayothes_ = Client(self.url).get_type('ns0:ArrayOtrosTributosType')
            Arrayothes = Arrayothes_(
                otroTributo=others
            )


        vattot = list()
        vatotot_ = Client(self.url).get_type('ns0:SubtotalIVAType')
        for itemdict in invoice.arraySubtotalesIVA:
            vattot.append(vatotot_(
                codigo=itemdict,
                importe=round(float(invoice.arraySubtotalesIVA[itemdict]),2),
            ))

        Arrayiva_ = Client(self.url).get_type('ns0:ArraySubtotalesIVAType')
        Arrayiva = Arrayiva_(
            subtotalIVA=vattot
        )

        last_invoice = self.get_last_number(pos, self._get_document_type(invoice))

        comprobanteRequest_ = Client(self.url).get_type('ns0:ComprobanteType')
        comprobanteRequest = comprobanteRequest_(
            codigoTipoComprobante=invoice.codigoTipoComprobante,
            numeroPuntoVenta=pos,
            numeroComprobante=last_invoice+1,
            codigoTipoDocumento=invoice.customer_document_type,
            numeroDocumento=invoice.customer_document_number,
            idImpositivo=invoice.idImpositivo,
            codigoRelacionEmisorReceptor=invoice.codigoRelacionEmisorReceptor,
            importeTotal=invoice.importeTotal,
            codigoMoneda=invoice.mon_id,
            cotizacionMoneda=invoice.mon_cotiz,
            arrayItems=ArrayItems,
            fechaEmision=invoice.fechaEmision,
            codigoTipoAutorizacion='E',
            #codigoAutorizacion=str(),
            codigoPais=invoice.codigoPais,
            domicilioReceptor=invoice.domicilioReceptor,
            importeGravado=invoice.importeGravado,
            importeNoGravado='0.00',
            importeExento='0.00',
            importeOtrosTributos=invoice.importeOtrosTributos,
            importeReintegro=invoice.importeReintegro,
            #observaciones=str(),
            arrayComprobantesAsociados=Arrayasoc,
            arrayOtrosTributos=Arrayothes,
            arraySubtotalesIVA=Arrayiva
        )

        AuthRequestType = Client(self.url).get_type('ns0:AuthRequestType')
        auth_request = AuthRequestType(
            token=self.accessToken.token,
            sign=self.accessToken.sign,
            cuitRepresentada=self.cuit
        )

        #AutorizarComprobanteRequest_ = Client(self.url).get_type('cts:AutorizarComprobanteRequestType')
        # AutorizarComprobanteRequest = AutorizarComprobanteRequest_(
        #     authRequest=auth_request,
        #     comprobanteRequest=comprobanteRequest,
        # )
        AutorizarComprobanteRequest = {
            'authRequest': {'token': self.accessToken.token, 'sign': self.accessToken.sign, 'cuitRepresentada': self.cuit},
            'comprobanteRequest': comprobanteRequest
        }

        #DEBUG
        # from lxml import etree as ET
        # client = Client(self.url)
        # node = client.create_message(client.service, 'autorizarComprobante',
        #     authRequest=auth_request,
        #     comprobanteRequest=comprobanteRequest,
        # )
        # tree = ET.ElementTree(node)
        # tree.write('/tmp/test.xml', pretty_print=True)

        return AutorizarComprobanteRequest

    def _create_auth_request(self):
        # CuitSimpleType = Client(self.url).get_type('ns0:CuitSimpleType')
        # CuitSimple = CuitSimpleType(self.cuit)
        AuthRequestType = Client(self.url).get_type('ns0:AuthRequestType')
        auth_request = AuthRequestType(
            token=self.accessToken.token,
            sign=self.accessToken.sign,
            cuitRepresentada=self.cuit
        )

        # auth_request = {
        #     'token': self.accessToken.token,
        #     'sign': self.accessToken.sign,
        #     'cuitRepresentada': self.cuit,
        # }
        return auth_request



authorization_urls = dict(
    homologation='https://wsaahomo.afip.gov.ar/ws/services/LoginCms?wsdl',
    production='https://wsaa.afip.gov.ar/ws/services/LoginCms?wsdl',
)

service_urls = dict(
    wstesimbrefiscal_homologation='https://wsaduhomoext.afip.gob.ar/diav2/wgestimbrefiscalelectronico/wgestimbrefiscalelectronico.asmx?WSDL',
    wstesimbrefiscal_production='https://wswadu.afip.gob.ar/DIAV2/wgestimbrefiscalelectronico/wgestimbrefiscalelectronico.asmx?WSDL',
    wsct_homologation='https://fwshomo.afip.gov.ar/wsct/CTService?WSDL',
    wsct_production='https://serviciosjava.afip.gob.ar/wsct/CTService?WSDL',
    wsbfev1_homologation='https://wswhomo.afip.gov.ar/wsbfev1/service.asmx?WSDL',
    wsbfev1_production='https://servicios1.afip.gov.ar/wsbfev1/service.asmx?WSDL',
    wsfev1_homologation='https://wswhomo.afip.gov.ar/wsfev1/service.asmx?wsdl',
    wsfev1_production='https://servicios1.afip.gov.ar/wsfev1/service.asmx?wsdl',
    wsfexv1_homologation='https://wswhomo.afip.gov.ar/wsfexv1/service.asmx?wsdl',
    wsfexv1_production='https://servicios1.afip.gov.ar/wsfexv1/service.asmx?wsdl',
    ws_sr_padron_a5_homologation='https://awshomo.afip.gov.ar/sr-padron/webservices/personaServiceA5?WSDL',
    ws_sr_padron_a5_production='https://aws.afip.gov.ar/sr-padron/webservices/personaServiceA5?WSDL',
)



from . import test_purchases_sales_presentation_line
from . import test_purchases_sales_presentation
from . import test_purchases_sales_vat_digital_book_line
from . import test_purchases_sales_vat_digital_book


# coding=utf-8

from .config import ClienteHttp

class ServicioCotArba(ClienteHttp):
    def __init__(self, user, password, tipo='homologation'):
        super().__init__(user, password, tipo)

    def presentar(self, filepath):
        with open(filepath, "rb") as archivo:
            file = {
                'file': (filepath, archivo, 'text/plain')
            }
            data = {
                "user": self.user,
                "password": self.password,
            }
            return self.post("/TransporteBienes/SeguridadCliente/presentarRemitos.do", files=file, data=data)
        

class CotRegistro(object):
    """
    Clase base para la generación de .txt según el diseño establecido por ARBA 
    https://www.arba.gov.ar/archivos/Publicaciones/nuevodiseniodearchivotxt.pdf
    """

    __slots__ = ['_tipoRegistro']

    def __init__(self, tipoRegistro):
        self._tipoRegistro = tipoRegistro 

    @property
    def tipoRegistro(self):
        return self._tipoRegistro

    def get_line_string(self):

        try:
            line_string = '|'.join(self.get_values())
        except TypeError:
            raise TypeError("La linea esta incompleta o es erronea")

        return line_string


    def get_values(self):
        raise NotImplementedError("Funcion get_registro no implementada para esta clase")


class CotHeader(CotRegistro):

    __slots__ = ['_cuitEmpresa']
     
    def __init__(self):
        super().__init__('01')
        self._cuitEmpresa = None

    @property
    def cuitEmpresa(self):
        return self._cuitEmpresa

    @cuitEmpresa.setter
    def cuitEmpresa(self, cuitEmpresa):
        self._cuitEmpresa = cuitEmpresa

    def get_values(self):
        return [self.tipoRegistro, self.cuitEmpresa]


class CotRemito(CotRegistro):

    __slots__ = ['_fechaEmision', '_codigoUnico', '_fechaSalidaTransporte', '_horaSalidaTransporte', '_sujetoGenerador', '_destinatarioConsumidorFinal',
                 '_destinatarioTipoDocumento', '_destinatarioDocumento', '_destinatarioCuit', '_destinatarioRazonSocial', '_destinatarioTenedor',
                 '_destinoDomicilioCalle', '_destinoDomicilioNumero', '_destinoDomicilioComple','_destinoDomicilioPiso', '_destinoDomicilioDto',
                 '_destinoDomicilioBarrio', '_destinoDomicilioCodigoPostal', '_destinoDomicilioLocalidad', '_destinoDomicilioProvincia', 
                 '_propioDestinoDomicilioCodigo','_entregaDomicilioOrigen', '_origenCuit', '_origenRazonSocial','_emisorTenedor', '_origenDomicilioCalle',
                 '_origenDomicilioNumero', '_origenDomicilioComple', '_origenDomicilioPiso', '_origenDomicilioDto', '_origenDomicilioBarrio',
                 '_origenDomicilioCodigoPostal', '_origenDomicilioLocalidad', '_origenDomicilioProvincia', '_transportistaCuit', '_tipoRecorrido', 
                 '_recorridoLocalidad', '_recorridoCalle', '_recorridoRuta', '_patenteVehiculo', '_patenteAcoplado', '_productoNoTermDev', '_importe']

    def __init__(self):
        super().__init__('02')
        self._fechaEmision = None
        self._codigoUnico = None 
        self._fechaSalidaTransporte = None
        self._horaSalidaTransporte = None
        self._sujetoGenerador = None
        self._destinatarioConsumidorFinal = None
        self._destinatarioTipoDocumento = None
        self._destinatarioDocumento = None
        self._destinatarioCuit = None
        self._destinatarioRazonSocial = None
        self._destinatarioTenedor = None
        self._destinoDomicilioCalle = None
        self._destinoDomicilioNumero = None
        self._destinoDomicilioComple = None
        self._destinoDomicilioPiso = None
        self._destinoDomicilioDto = None
        self._destinoDomicilioBarrio = None
        self._destinoDomicilioCodigoPostal = None
        self._destinoDomicilioLocalidad = None
        self._destinoDomicilioProvincia = None
        self._entregaDomicilioOrigen = None
        self._origenCuit = None
        self._origenRazonSocial = None
        self._emisorTenedor = None
        self._origenDomicilioCalle = None
        self._origenDomicilioNumero = None
        self._origenDomicilioComple = None
        self._origenDomicilioPiso = None
        self._origenDomicilioDto = None
        self._origenDomicilioBarrio = None
        self._origenDomicilioCodigoPostal = None
        self._origenDomicilioLocalidad = None
        self._origenDomicilioProvincia = None
        self._transportistaCuit = None
        self._tipoRecorrido = None
        self._recorridoLocalidad = None
        self._recorridoCalle = None
        self._recorridoRuta = None
        self._patenteVehiculo = None
        self._patenteAcoplado = None
        self._productoNoTermDev = None
        self._importe = None

    @property
    def fechaEmision(self):
        return self._fechaEmision

    @fechaEmision.setter
    def fechaEmision(self, fechaEmision):
        self._fechaEmision = fechaEmision

    @property
    def codigoUnico(self):
        return self._codigoUnico

    @codigoUnico.setter
    def codigoUnico(self, codigoUnico):
        self._codigoUnico = codigoUnico

    @property
    def fechaSalidaTransporte(self):
        return self._fechaSalidaTransporte

    @fechaSalidaTransporte.setter
    def fechaSalidaTransporte(self, fechaSalidaTransporte):
        self._fechaSalidaTransporte = fechaSalidaTransporte

    @property
    def horaSalidaTransporte(self):
        return self._horaSalidaTransporte

    @horaSalidaTransporte.setter
    def horaSalidaTransporte(self, horaSalidaTransporte):
        self._horaSalidaTransporte = horaSalidaTransporte

    @property
    def sujetoGenerador(self):
        return self._sujetoGenerador

    @sujetoGenerador.setter
    def sujetoGenerador(self, sujetoGenerador):
        self._sujetoGenerador = sujetoGenerador

    @property
    def destinatarioConsumidorFinal(self):
        return self._destinatarioConsumidorFinal

    @destinatarioConsumidorFinal.setter
    def destinatarioConsumidorFinal(self, destinatarioConsumidorFinal):
        self._destinatarioConsumidorFinal = destinatarioConsumidorFinal

    @property
    def destinatarioTipoDocumento(self):
        return self._destinatarioTipoDocumento

    @destinatarioTipoDocumento.setter
    def destinatarioTipoDocumento(self, destinatarioTipoDocumento):
        self._destinatarioTipoDocumento = destinatarioTipoDocumento

    @property
    def destinatarioDocumento(self):
        return self._destinatarioDocumento

    @destinatarioDocumento.setter
    def destinatarioDocumento(self, destinatarioDocumento):
        self._destinatarioDocumento = destinatarioDocumento

    @property
    def destinatarioCuit(self):
        return self._destinatarioCuit

    @destinatarioCuit.setter
    def destinatarioCuit(self, destinatarioCuit):
        self._destinatarioCuit = destinatarioCuit

    @property
    def destinatarioRazonSocial(self):
        return self._destinatarioRazonSocial

    @destinatarioRazonSocial.setter
    def destinatarioRazonSocial(self, destinatarioRazonSocial):
        self._destinatarioRazonSocial = destinatarioRazonSocial

    @property
    def destinatarioTenedor(self):
        return self._destinatarioTenedor

    @destinatarioTenedor.setter
    def destinatarioTenedor(self, destinatarioTenedor):
        self._destinatarioTenedor = destinatarioTenedor

    @property
    def destinoDomicilioCalle(self):
        return self._destinoDomicilioCalle

    @destinoDomicilioCalle.setter
    def destinoDomicilioCalle(self, destinoDomicilioCalle):
        self._destinoDomicilioCalle = destinoDomicilioCalle

    @property
    def destinoDomicilioNumero(self):
        return self._destinoDomicilioNumero

    @destinoDomicilioNumero.setter
    def destinoDomicilioNumero(self, destinoDomicilioNumero):
        self._destinoDomicilioNumero = destinoDomicilioNumero

    @property
    def destinoDomicilioComple(self):
        return self._destinoDomicilioComple

    @destinoDomicilioComple.setter
    def destinoDomicilioComple(self, destinoDomicilioComple):
        self._destinoDomicilioComple = destinoDomicilioComple

    @property
    def destinoDomicilioPiso(self):
        return self._destinoDomicilioPiso

    @destinoDomicilioPiso.setter
    def destinoDomicilioPiso(self, destinoDomicilioPiso):
        self._destinoDomicilioPiso = destinoDomicilioPiso

    @property
    def destinoDomicilioDto(self):
        return self._destinoDomicilioDto

    @destinoDomicilioDto.setter
    def destinoDomicilioDto(self, destinoDomicilioDto):
        self._destinoDomicilioDto = destinoDomicilioDto

    @property
    def destinoDomicilioBarrio(self):
        return self._destinoDomicilioBarrio

    @destinoDomicilioBarrio.setter
    def destinoDomicilioBarrio(self, destinoDomicilioBarrio):
        self._destinoDomicilioBarrio = destinoDomicilioBarrio

    @property
    def destinoDomicilioCodigoPostal(self):
        return self._destinoDomicilioCodigoPostal

    @destinoDomicilioCodigoPostal.setter
    def destinoDomicilioCodigoPostal(self, destinoDomicilioCodigoPostal):
        self._destinoDomicilioCodigoPostal = destinoDomicilioCodigoPostal

    @property
    def destinoDomicilioLocalidad(self):
        return self._destinoDomicilioLocalidad

    @destinoDomicilioLocalidad.setter
    def destinoDomicilioLocalidad(self, destinoDomicilioLocalidad):
        self._destinoDomicilioLocalidad = destinoDomicilioLocalidad

    @property
    def destinoDomicilioProvincia(self):
        return self._destinoDomicilioProvincia

    @destinoDomicilioProvincia.setter
    def destinoDomicilioProvincia(self, destinoDomicilioProvincia):
        self._destinoDomicilioProvincia = destinoDomicilioProvincia

    @property
    def propioDestinoDomicilioCodigo(self):
        return self._propioDestinoDomicilioCodigo

    @propioDestinoDomicilioCodigo.setter
    def propioDestinoDomicilioCodigo(self, propioDestinoDomicilioCodigo):
        self._propioDestinoDomicilioCodigo = propioDestinoDomicilioCodigo

    @property
    def entregaDomicilioOrigen(self):
        return self._entregaDomicilioOrigen

    @entregaDomicilioOrigen.setter
    def entregaDomicilioOrigen(self, entregaDomicilioOrigen):
        self._entregaDomicilioOrigen = entregaDomicilioOrigen

    @property
    def origenCuit(self):
        return self._origenCuit

    @origenCuit.setter
    def origenCuit(self, origenCuit):
        self._origenCuit = origenCuit

    @property
    def origenRazonSocial(self):
        return self._origenRazonSocial

    @origenRazonSocial.setter
    def origenRazonSocial(self, origenRazonSocial):
        self._origenRazonSocial = origenRazonSocial

    @property
    def emisorTenedor(self):
        return self._emisorTenedor

    @emisorTenedor.setter
    def emisorTenedor(self, emisorTenedor):
        self._emisorTenedor = emisorTenedor

    @property
    def origenDomicilioCalle(self):
        return self._origenDomicilioCalle

    @origenDomicilioCalle.setter
    def origenDomicilioCalle(self, origenDomicilioCalle):
        self._origenDomicilioCalle = origenDomicilioCalle

    @property
    def origenDomicilioNumero(self):
        return self._origenDomicilioNumero

    @origenDomicilioNumero.setter
    def origenDomicilioNumero(self, origenDomicilioNumero):
        self._origenDomicilioNumero = origenDomicilioNumero

    @property
    def origenDomicilioComple(self):
        return self._origenDomicilioComple

    @origenDomicilioComple.setter
    def origenDomicilioComple(self, origenDomicilioComple):
        self._origenDomicilioComple = origenDomicilioComple

    @property
    def origenDomicilioPiso(self):
        return self._origenDomicilioPiso

    @origenDomicilioPiso.setter
    def origenDomicilioPiso(self, origenDomicilioPiso):
        self._origenDomicilioPiso = origenDomicilioPiso

    @property
    def origenDomicilioDto(self):
        return self._origenDomicilioDto

    @origenDomicilioDto.setter
    def origenDomicilioDto(self, origenDomicilioDto):
        self._origenDomicilioDto = origenDomicilioDto

    @property
    def origenDomicilioBarrio(self):
        return self._origenDomicilioBarrio

    @origenDomicilioBarrio.setter
    def origenDomicilioBarrio(self, origenDomicilioBarrio):
        self._origenDomicilioBarrio = origenDomicilioBarrio

    @property
    def origenDomicilioCodigoPostal(self):
        return self._origenDomicilioCodigoPostal

    @origenDomicilioCodigoPostal.setter
    def origenDomicilioCodigoPostal(self, origenDomicilioCodigoPostal):
        self._origenDomicilioCodigoPostal = origenDomicilioCodigoPostal

    @property
    def origenDomicilioLocalidad(self):
        return self._origenDomicilioLocalidad

    @origenDomicilioLocalidad.setter
    def origenDomicilioLocalidad(self, origenDomicilioLocalidad):
        self._origenDomicilioLocalidad = origenDomicilioLocalidad

    @property
    def origenDomicilioProvincia(self):
        return self._origenDomicilioProvincia

    @origenDomicilioProvincia.setter
    def origenDomicilioProvincia(self, origenDomicilioProvincia):
        self._origenDomicilioProvincia = origenDomicilioProvincia

    @property
    def transportistaCuit(self):
        return self._transportistaCuit

    @transportistaCuit.setter
    def transportistaCuit(self, transportistaCuit):
        self._transportistaCuit = transportistaCuit

    @property
    def tipoRecorrido(self):
        return self._tipoRecorrido

    @tipoRecorrido.setter
    def tipoRecorrido(self, tipoRecorrido):
        self._tipoRecorrido = tipoRecorrido

    @property
    def recorridoLocalidad(self):
        return self._recorridoLocalidad

    @recorridoLocalidad.setter
    def recorridoLocalidad(self, recorridoLocalidad):
        self._recorridoLocalidad = recorridoLocalidad

    @property
    def recorridoCalle(self):
        return self._recorridoCalle

    @recorridoCalle.setter
    def recorridoCalle(self, recorridoCalle):
        self._recorridoCalle = recorridoCalle

    @property
    def recorridoRuta(self):
        return self._recorridoRuta

    @recorridoRuta.setter
    def recorridoRuta(self, recorridoRuta):
        self._recorridoRuta = recorridoRuta

    @property
    def patenteVehiculo(self):
        return self._patenteVehiculo

    @patenteVehiculo.setter
    def patenteVehiculo(self, value):
        self._patenteVehiculo = value

    @property
    def patenteAcoplado(self):
        return self._patenteAcoplado

    @patenteAcoplado.setter
    def patenteAcoplado(self, patenteAcoplado):
        self._patenteAcoplado = patenteAcoplado

    @property
    def productoNoTermDev(self):
        return self._productoNoTermDev

    @productoNoTermDev.setter
    def productoNoTermDev(self, productoNoTermDev):
        self._productoNoTermDev = productoNoTermDev

    @property
    def importe(self):
        return self._importe

    @importe.setter
    def importe(self, importe):
        self._importe = importe

    def get_values(self):
        return [
            self.tipoRegistro,
            self.fechaEmision,
            self.codigoUnico,
            self.fechaSalidaTransporte,
            self.horaSalidaTransporte,
            self.sujetoGenerador,
            self.destinatarioConsumidorFinal,
            self.destinatarioTipoDocumento,
            self.destinatarioDocumento,
            self.destinatarioCuit,
            self.destinatarioRazonSocial,
            self.destinatarioTenedor,
            self.destinoDomicilioCalle,
            self.destinoDomicilioNumero,
            self.destinoDomicilioComple,
            self.destinoDomicilioPiso,
            self.destinoDomicilioDto,
            self.destinoDomicilioBarrio,
            self.destinoDomicilioCodigoPostal,
            self.destinoDomicilioLocalidad,
            self.destinoDomicilioProvincia,
            self.propioDestinoDomicilioCodigo,
            self.entregaDomicilioOrigen,
            self.origenCuit,
            self.origenRazonSocial,
            self.emisorTenedor,
            self.origenDomicilioCalle,
            self.origenDomicilioNumero,
            self.origenDomicilioComple,
            self.origenDomicilioPiso,
            self.origenDomicilioDto,
            self.origenDomicilioBarrio,
            self.origenDomicilioCodigoPostal,
            self.origenDomicilioLocalidad,
            self.origenDomicilioProvincia,
            self.transportistaCuit,
            self.tipoRecorrido,
            self.recorridoLocalidad,
            self.recorridoCalle,
            self.recorridoRuta,
            self.patenteVehiculo,
            self.patenteAcoplado,
            self.productoNoTermDev,
            self.importe
        ]

class CotProducto(CotRegistro):
    __slots__ = ['_codigoUnicoProducto', '_arbaCodigoUnidadMedida', '_cantidad', '_propioCodigoProducto', '_propioDescripcionProducto',
                 '_propioDescripcionUnidadMedida', '_cantidadAjustada']
    def __init__(self):
        super().__init__('03')
        self._codigoUnicoProducto = None
        self._arbaCodigoUnidadMedida = None
        self._cantidad = None
        self._propioCodigoProducto = None
        self._propioDescripcionProducto = None
        self._propioDescripcionUnidadMedida = None
        self._cantidadAjustada = None

    @property
    def codigoUnicoProducto(self):
        return self._codigoUnicoProducto

    @codigoUnicoProducto.setter 
    def codigoUnicoProducto(self, codigoUnicoProducto):
        self._codigoUnicoProducto = codigoUnicoProducto

    @property
    def arbaCodigoUnidadMedida(self):
        return self._arbaCodigoUnidadMedida

    @arbaCodigoUnidadMedida.setter 
    def arbaCodigoUnidadMedida(self, arbaCodigoUnidadMedida):
        self._arbaCodigoUnidadMedida = arbaCodigoUnidadMedida

    @property
    def cantidad(self):
        return self._cantidad

    @cantidad.setter
    def cantidad(self, cantidad):
        self._cantidad = cantidad

    @property
    def propioCodigoProducto(self):
        return self._propioCodigoProducto

    @propioCodigoProducto.setter 
    def propioCodigoProducto(self, propioCodigoProducto):
        self._propioCodigoProducto = propioCodigoProducto

    @property
    def propioDescripcionProducto(self):
        return self._propioDescripcionProducto

    @propioDescripcionProducto.setter 
    def propioDescripcionProducto(self, propioDescripcionProducto):
        self._propioDescripcionProducto = propioDescripcionProducto[:40]

    @property
    def propioDescripcionUnidadMedida(self):
        return self._propioDescripcionUnidadMedida

    @propioDescripcionUnidadMedida.setter
    def propioDescripcionUnidadMedida(self, propioDescripcionUnidadMedida):
        self._propioDescripcionUnidadMedida = propioDescripcionUnidadMedida

    @property
    def cantidadAjustada(self):
        return self._cantidadAjustada
    
    @cantidadAjustada.setter
    def cantidadAjustada(self, cantidadAjustada):
        self._cantidadAjustada = cantidadAjustada 

    def get_values(self):
        return [
            self.tipoRegistro,
            self.codigoUnicoProducto,
            self.arbaCodigoUnidadMedida,
            self.cantidad,
            self.propioCodigoProducto,
            self.propioDescripcionProducto,
            self.propioDescripcionUnidadMedida,
            self.cantidadAjustada
        ]


class CotFooter(CotRegistro):
    __slots__ = ['_cantidadTotalRemitos']
    def __init__(self):
        super().__init__('04')
        self._cantidadTotalRemitos = None

    @property
    def cantidadTotalRemitos(self):
        return self._cantidadTotalRemitos
    
    @cantidadTotalRemitos.setter
    def cantidadTotalRemitos(self, cantidadTotalRemitos):
        self._cantidadTotalRemitos = cantidadTotalRemitos

    def get_values(self):
        return [
            self.tipoRegistro,
            self.cantidadTotalRemitos
        ]

class GeneradorArchivoCot:
    def __init__(self):
        self.registros = []

    def agregar_registro(self, registro):
        self.registros.append(registro)

    def generar_archivo(self, name=''):
        contenido = ""
        for registro in self.registros:
            contenido += registro.get_line_string() + "\n"
        with open(f'/tmp/{name}.txt', "w") as archivo:
            archivo.write(contenido)
            return archivo 





# -*- coding: utf-8 -*-
class Contributor:

    @classmethod
    def is_valid_cuit(cls, cuit):
        """
        :param cuit: Cuit a validar
        :return: Falso si no es valido
        """

        try:
            int(cuit)
        except ValueError:
            return False

        cuit = str(cuit)

        if len(cuit) != 11:
            return False

        numbers = [5, 4, 3, 2, 7, 6, 5, 4, 3, 2]

        var1 = 0
        for i in range(10):
            var1 = var1 + int(cuit[i]) * numbers[i]
        var3 = 11 - var1 % 11

        if var3 == 11:
            var3 = 0
        if var3 == 10:
            return False
        if var3 == int(cuit[10]):
            return True

        return False


# coding=utf-8

import requests
import hashlib
import xml.etree.cElementTree as et
from dateutil.relativedelta import relativedelta
from . import config
from os.path import abspath


def create_file(path, date):
    root = et.Element("DESCARGA-PADRON")
    et.SubElement(root, 'fechaDesde').text = date.replace(day=1).strftime('%Y%m%d')
    et.SubElement(root, 'fechaHasta').text = (date.replace(day=1) + relativedelta(months=1, days=-1)).strftime('%Y%m%d')
    tree = et.ElementTree(root)
    tree.write(path)


def get_md5(path):
    padron_request_file = open(path, 'r')
    content = padron_request_file.read()
    return hashlib.md5(content.encode()).hexdigest()


class IIBBPadron(object):

    """
    Descarga de padrón según especificaciones
    http://www.arba.gov.ar/Domicilio_Electronico/VerPDF.asp?param=ES
    """
    def __init__(self, user, password, homologation=False):
        self.user = user
        self.password = password
        self.url = config.authorization_urls['homologation'] if homologation else \
            config.authorization_urls['production']

    def get_padron(self, date, request_path='/tmp/padron_request.xml', padron_path='/tmp/padron.zip'):
        """
        Descarga el padron otorgado por ARBA
        :param date: Fecha de la cual se desea obtener el padron
        :param request_path: Ruta donde se creará el archivo de request.
        :param padron_path: Ruta donde se descargará el padrón.
        """
        
        create_file(request_path, date)
        filename = get_md5(request_path)
        
        with open(request_path, 'r') as fileaenviar, open(padron_path, 'wb') as filearecibir:
            r = requests.post(
                self.url,
                data={'user': self.user, 'password': self.password},
                files={'file': ('DFEServicioDescargaPadron_'+filename+'.xml', fileaenviar, 'text/xml')},
            )
            filearecibir.write(r.content)

        return abspath(padron_path)



# -*- coding: utf-8 -*-
class AfipError:
    
    @classmethod
    def parse_error(cls, error):
        return Exception("Error {}: {}".format(
            error.arrayErrores.codigoDescripcion[0].codigo,
            error.arrayErrores.codigoDescripcion[0].descripcion.encode('utf-8')),
        )
